{-# LANGUAGE OverloadedStrings #-}
module MyLib (defaultMain) where

import Control.Exception (bracket)
import Control.Lens
import Control.Monad (void)
import qualified Data.Aeson as A
import qualified Data.ByteString.Lazy as LB
import Data.Config
import Data.Foldable (for_)
import Data.Functor.Contravariant ((>$<))
import Data.Text (Text)
import qualified Data.Text.Strict.Lens as T
import Network.AMQP
import Network.Reddit (publishEntries, RedditId, messageIdentifier)
import Network.Wreq.Session (newSession)
import Publish (Publish(..))
import qualified Database.SQLite.Simple as SQL
import qualified Membership

sqlRecorder :: SQL.Connection -> Publish IO RedditId
sqlRecorder conn = Publish $ Membership.recordSeen conn

amqpPublisher :: Channel -> Text -> Publish IO LB.ByteString
amqpPublisher channel exchange = Publish $ \lbs ->
  void $ publishMsg channel exchange routingKey (message lbs)
  where
    routingKey = "doesn't matter on fanout"
    message lbs = newMsg
      { msgBody = lbs
      , msgDeliveryMode = Just Persistent
      }

defaultMain :: IO ()
defaultMain = do
  conf <- readConfig "./config.dhall"
  let rabbitConnect = openConnection
                  (conf ^. amqp . host . T.unpacked)
                  (conf ^. amqp . vhost)
                  (conf ^. amqp . username)
                  (conf ^. amqp . password)
  bracket rabbitConnect closeConnection $ \conn -> do
    SQL.withConnection (conf ^. sqlite) $ \sqlConn -> do
      SQL.execute_ sqlConn "create table if not exists membership (reddit_id primary key)"
      chan <- openChannel conn
      declareExchange chan newExchange { exchangeName = "reddit_posts", exchangeType = "fanout" }
      sess <- newSession
      let encoder = amqpPublisher chan "reddit_posts"
          recorder = sqlRecorder sqlConn
          publisher = (A.encode >$< encoder) <> (messageIdentifier >$< recorder)
      for_ (conf ^. fetchers) $ \fetcher -> do
        print fetcher
        publishEntries publisher sess fetcher
